// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//
//-----------------------------------------------------------------------------

static const char
rcsid[] = "$Id: m_bbox.c,v 1.1 1997/02/03 22:45:10 b1 Exp $";

#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <stdarg.h>

#ifndef _MSC_VER
#include <sys/time.h>
#include <unistd.h>
#endif

#include "doomdef.h"
#include "info.h"
#include "m_misc.h"
#include "i_video.h"
#include "i_sound.h"

#include "d_net.h"
#include "g_game.h"

#ifndef USE_X11
#include "../TerminalDoom.h"
#endif

#ifdef __GNUG__
#pragma implementation "i_system.h"
#endif
#include "i_system.h"



int	mb_used = 6;


void
I_Tactile
( int	on,
  int	off,
  int	total )
{
  // UNUSED.
  on = off = total = 0;
}

ticcmd_t	emptycmd;
ticcmd_t*	I_BaseTiccmd(void)
{
    return &emptycmd;
}


int  I_GetHeapSize (void)
{
    return mb_used*1024*1024;
}

byte* I_ZoneBase (int*	size)
{
    *size = mb_used*1024*1024;
    return (byte *) malloc (*size);
}



//
// I_GetTime
// returns time in 1/70th second tics
//
int  I_GetTime (void)
{
#ifndef _MSC_VER

#if 0
    // Try using the Doom3 engine timer.
    return int(TICRATE*TheTerminal::GetTimeSeconds());
#else//0
    struct timeval	tp;
    struct timezone	tzp;
    int			newtics;
    static int		basetime=0;
  
    gettimeofday(&tp, &tzp);
    if (!basetime)
	basetime = tp.tv_sec;
    newtics = (tp.tv_sec-basetime)*TICRATE + tp.tv_usec*TICRATE/1000000;
    return newtics;
#endif

#else
	// Try using the Doom3 engine timer on Windows? 
	//return (TICRATE*TheTerminal::GetTimeSeconds());

	//return 1; // bk  - this requires singletics

	LARGE_INTEGER frequency;
	BOOL result = QueryPerformanceFrequency( &frequency);
	LARGE_INTEGER counter;
	result &= QueryPerformanceCounter( &counter );
	assert( result );

	static LARGE_INTEGER current = { 0L };
	if ( current.QuadPart==0L )
		current = counter;
	LARGE_INTEGER delta = { 0L };
	delta.QuadPart= (current.QuadPart-counter.QuadPart);
		
	const int newtics = ((delta.QuadPart/frequency.QuadPart)*TICRATE);
	return newtics;
#endif
}



//
// I_Init
//
void I_Init (void)
{
    I_InitSound();
    //  I_InitGraphics();
}

//
// I_Quit
//
void I_Quit (void)
{
#ifdef USE_X11
    D_QuitNetGame ();
    I_ShutdownSound();
    I_ShutdownMusic();
    M_SaveDefaults ();
    I_ShutdownGraphics();
    exit(0);
#endif
}

void I_WaitVBL(int count)
{
#ifdef USE_X11
#ifdef SGI
    sginap(1);                                           
#else
#ifdef SUN
    sleep(0);
#else
    usleep (count * (1000000/70) );                                
#endif
#endif
#endif
}

void I_BeginRead(void)
{
}

void I_EndRead(void)
{
}

byte*	I_AllocLow(int length)
{
    byte*	mem;
        
    mem = (byte *)malloc (length);
    memset (mem,0,length);
    return mem;
}


//
// I_Error
//
extern boolean demorecording;

void I_Error (char *error, ...)
{
#ifdef USE_X11
    va_list	argptr;

    // Message first.
    va_start (argptr,error);
    fprintf (stderr, "Error: ");
    vfprintf (stderr,error,argptr);
    fprintf (stderr, "\n");
    va_end (argptr);

    fflush( stderr );

    // Shutdown. Here might be other errors.
    if (demorecording)
		G_CheckDemoStatus();

    D_QuitNetGame ();
    I_ShutdownGraphics();
    
    exit(-1);
#else
	va_list	argptr;
	char	buffer[1024];

    // Message first.
    va_start (argptr,error);
    vsprintf (buffer,error,argptr);
	TheTerminal::Printf(buffer);
    va_end (argptr);

	  // Shutdown. Here might be other errors.
    if (demorecording)
		G_CheckDemoStatus();

    D_QuitNetGame ();
    I_ShutdownGraphics();
#endif
}

void I_Printf (char *format, ...)
{
	va_list	argptr;
	char	buffer[1024];

    // Message first.
    va_start (argptr,format);
    vsprintf (buffer,format,argptr);
#ifndef USE_X11
    TheTerminal::Printf(buffer);
#else
    fprintf( stderr, buffer );
#endif
    va_end (argptr);
}
